import datetime
import uuid


class Header:
    """
    Provides the meta information needed to increase traceability.
    """

    id = ''
    source = ''
    timestamp = ''
    type = ''

    def __init__(self, source=None, event=None):
        """
        Id and timestamps are added automatically. The id is an UUID which
        identifies the current message, so it can be traced across services.
        :param source: The source which the message is originated from.
        :param event: The name of the event which is produced.
        """
        self.id = str(uuid.uuid4())
        self.timestamp = datetime.datetime.now().isoformat()
        self.source = source
        self.type = event




import json
import logging
from typing import Tuple

import confluent_kafka as kafka
from confluent_kafka.cimpl import Message
from eventcore import Consumer
from eventcore.exceptions import FatalConsumerError

log = logging.getLogger(__name__)


class KafkaConsumer(Consumer):
    """
    Consume from a Kafka queue.
    :param servers: list of brokers to consume from.
    :param group_id: identifier for this consumer.
    :param topics: list of topics to consume from.
    """

    def __init__(self, servers, group_id, topics, **kwargs):
        # Parse the servers to ensure it's a comma-separated string.
        if isinstance(servers, list):
            servers = ','.join(servers)
        self.kafka_consumer = kafka.Consumer({
            'bootstrap.servers': servers,
            'group.id': group_id,
            **kwargs
        })
        # Parse the topics to ensure it's a list.
        if isinstance(topics, str):
            topics = topics.split(',')

        self.kafka_consumer.subscribe(topics)

    def consume(self):
        while True:
            self.poll_and_process()

    def poll_and_process(self):
        message = self.kafka_consumer.poll()
        if not self.is_valid_message(message):
            return
        subject, message_body = self.parse_message(message)
        self.process_event(
            name=message_body.get('event'),
            subject=subject,
            data=message_body.get('data'))

    @staticmethod
    def is_valid_message(message: Message):
        if not message:
            return False
        if message.error():
            # PARTITION_EOF error can be ignored.
            if message.error().code() == kafka.KafkaError._PARTITION_EOF:
                return False
            else:
                raise kafka.KafkaException(message.error())
        return True

    @staticmethod
    def parse_message(message: Message) -> Tuple[str, dict]:
        subject, message_body = None, None
        try:
            message_body = json.loads(message.value())
        except TypeError:
            message_body = json.loads(message.value().decode('utf-8'))
        except:
            log.error('@KafkaConsumer.consume Exception:', exc_info=True)
        try:
            subject = message.key().decode('utf-8')
        except AttributeError:
            subject = message.key()

        return subject, message_body


class BlockingKafkaConsumer(KafkaConsumer):
    """Consumer for Kafka topics, blocks when a message cannot be processed."""

    def __init__(self, servers, group_id, topics, **kwargs):
        kwargs['enable.auto.commit'] = False
        kwargs['auto.offset.reset'] = "smallest"  # Start from first failed.
        super().__init__(servers, group_id, topics, **kwargs)

    def poll_and_process(self):
        message = self.kafka_consumer.poll()
        if not self.is_valid_message(message):
            return
        subject, message_body = self.parse_message(message)
        try:
            self.process_event(
                name=message_body.get('event'),
                subject=subject,
                data=message_body.get('data'))
            self.kafka_consumer.commit(message)
        except BaseException:
            raise FatalConsumerError(
                "Message with body {} could not be processed and blocks "
                "the consumer. Manual action required.".format(message_body))



import json
import confluent_kafka as kafka
from eventcore_kafka.header import Header

from eventcore import Producer


class KafkaProducer(Producer):
    """
    Produce to a Kafka queue.
    :param servers: list of brokers to consume from.
    """

    source = ''

    def __init__(self, servers, source=None):
        """
        Initialize the producer for Kafka
        :param servers: The host and port of where Kafka runs.
        :param source: The source of the application which is producing the
        messages.
        """

        self.source = source

        # Parse the servers to ensure it's a comma-separated string.
        if isinstance(servers, list):
            servers = ','.join(servers)
        self.kafka_producer = kafka.Producer({
            'bootstrap.servers': servers
        })

    def get_headers(self, event):
        """
        Creates an header which is added to Kafka
        :param event: the name of the event e.g user.created
        :return: a json serialized representation of the header
        """
        header = Header(source=self.source, event=event)
        return header.__dict__

    def produce(self, topic, event, subject, data):
        message_body = json.dumps({
            'event': event,
            'data': data
        })
        self.kafka_producer.produce(topic=topic,
                                    key=subject,
                                    value=message_body,
                                    headers=self.get_headers(event))



import setuptools


setuptools.setup(
    name='eventcore-kafka',
    version='0.3.3rc2',
    description='Produce and consume events with Kafka.',
    author='Maikel van den Boogerd',
    author_email='maikelboogerd@gmail.com',
    url='https://github.com/maikelboogerd/eventcore-kafka',
    keywords=['event', 'kafka', 'producer', 'consumer'],
    packages=['eventcore_kafka'],
    install_requires=['eventcore', 'confluent-kafka'],
    license='MIT',
    zip_safe=False
)
